import json
import requests
import io
from typing import Dict, Union


class APIRequest:
    API_URL = 'https://api.midjourneyapi.io/v2/'

    def __init__(self, api_key: str, callback_uri: str = None):
        self.api_key = api_key
        self.callback_uri = callback_uri

    @staticmethod
    def send_request(endpoint: str, data: dict, headers: dict, files: dict = None) -> Union[Dict, str]:
        url = APIRequest.API_URL + endpoint
        if files is not None:
            response = requests.post(
                url, data=data, files=files, headers=headers)
        else:
            response = requests.post(url, data=data, headers=headers)

        if response.status_code == 200:
            return response.json()
        else:
            print(response)
            return response.text

    def describe_image(self, image_file_path: str, filename: str) -> Dict:
        endpoint = 'describe'
        files = [
            ('image', (filename, open(
                image_file_path, 'rb'), 'image/png'))
        ]

        data = {}
        if self.callback_uri != "":
            data['callbackURL'] = self.callback_uri

        headers = {'Authorization': self.api_key}
        return APIRequest.send_request(endpoint, data, headers, files)

    def get_result(self, task_id: str) -> Dict:
        endpoint = 'result'
        data = json.dumps({'taskId': task_id})
        headers = {'Authorization': self.api_key,
                   'Content-Type': 'application/json'}
        return APIRequest.send_request(endpoint, data, headers)

    def upscale_image(self, task_id: str, position: str) -> Dict:
        endpoint = 'upscale'
        data = {'taskId': task_id, 'position': position}
        headers = {'Authorization': self.api_key}
        return APIRequest.send_request(endpoint, data, headers)

    def imagine(self, prompt: str) -> Dict:
        endpoint = 'imagine'
        data = {'prompt': prompt}
        if self.callback_uri != "":
            data['callbackURL'] = self.callback_uri

        headers = {'Authorization': self.api_key}
        return APIRequest.send_request(endpoint, data, headers)

    def seed(self, task_id: str) -> Dict:
        endpoint = 'seed'
        data = {
            'taskId': task_id
        }

        if self.callback_uri != "":
            data['callbackURL'] = self.callback_uri

        headers = {'Authorization': self.api_key}
        return APIRequest.send_request(endpoint, data, headers)



from setuptools import setup, find_packages
import codecs
import os

here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()

VERSION = '1.0.0'
DESCRIPTION = 'MidjourneyPy allows developers to easily use midjourney api without complications.'
LONG_DESCRIPTION = '''The MidjourneyPy APIRequest library provides a simplified and convenient way for developers to interact with the Midjourney API. With this library, developers can easily integrate Midjourney's powerful image processing capabilities into their applications without dealing with the complexities of manual API calls.'''

# Setting up
setup(
    name="Midjourney-py",
    version=VERSION,
    author="Ajaga Abdulbasit (Code  Ninja)",
    author_email="basitng2004@gmail.com",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=long_description,
    packages=find_packages(),
    install_requires=['requests'],
    keywords=['python', 'ai', 'midjourney',
              'image', 'request', 'stable', 'stable difusion', 'difusion'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)



from typing import Dict
import time
from midjourney.irequest import APIRequest


class Midjourney:
    def __init__(self, api_key: str, callback_uri: str):
        self.api_request = APIRequest(api_key, callback_uri)

    def result(self, seed: Dict) -> Dict:
        task_id = seed.get('taskId')

        if not task_id:
            return {'status': 'error', 'message': 'Invalid seed data'}

        print("Running task....")
        while True:
            try:
                response = self.api_request.get_result(task_id)
            except Exception as e:
                return {'status': 'error', 'message': str(e)}

            if response and isinstance(response, dict):
                if 'imageURL' in response:
                    print("Completed task....")
                    return {'status': 'completed', 'imageUrl': response['imageURL']}
                elif 'seed' in response:
                    print("Completed task....")
                    return {'status': 'completed', 'seed': response['seed']}
                elif 'content' in response:
                    print("Completed task....")
                    return {'status': 'completed', 'content': response['content']}
                elif response.get('status') == 'running':
                    percentage = response.get('percentage')
                    print(f"Task is {percentage}% complete.")
                elif response.get('status') in ['pending', 'waiting-to-start', 'unknown', {}, 'failed-please-resubmit']:
                    print("Task is pending. Waiting for it to start...")
                    time.sleep(2)
                    continue
            else:
                print("Waiting for task to start...")
                time.sleep(2)
                continue

            if response.get('status') not in ['running', 'pending', 'unknown']:
                break

    def imagine(self, prompt: str) -> Dict:
        return self.api_request.imagine(prompt)

    def upscale(self, task_id: str, position: str) -> Dict:
        return self.api_request.upscale_image(task_id, position)

    def seed(self, task_id: str) -> Dict:
        return self.api_request.seed(task_id)

    def describe(self, image_path: str, filename: str) -> Dict:
        return self.api_request.describe_image(image_file_path=image_path, filename=filename)
